export default ({
  region,
  project,
  network,
  host,
  procedure,
  memory,
  timeout,
  baseNetwork,
  operationHash,
  containerRegistery,
  envUriSpecifier,
  serviceName,
  rolesBucket,
  secretBucket,
  secretBucketKeyLocation,
  secretBucketKeyRing,
  computeUrlId,
  redisIp,
  extension,
  nodeEnv,
  env = "",
  labels = "",
  allowUnauthenticated = false,
} = {}) => {
  return {
    name: "gcr.io/cloud-builders/gcloud",
    args: [
      "alpha",
      "run",
      "deploy",
      `${serviceName}`,
      `--image=${containerRegistery}/${procedure}${
        extension ? `.${extension}` : ""
      }`,
      "--platform=managed",
      `--memory=${memory}`,
      `--timeout=${timeout}`,
      `--vpc-connector=${region}-network`,
      `--vpc-egress=all`,
      ...(allowUnauthenticated ? ["--allow-unauthenticated"] : []),
      `--project=${project}`,
      `--region=${region}`,
      `--set-env-vars=${Object.entries({
        NODE_ENV: nodeEnv,
        NETWORK: `${envUriSpecifier}${network}`,
        BASE_NETWORK: baseNetwork,
        HOST: host,
        PROCEDURE: procedure,
        OPERATION_HASH: operationHash,
        SERVICE_NAME: serviceName,
        GCP_PROJECT: project,
        GCP_REGION: region,
        ...(redisIp && { REDIS_IP: redisIp }),
        GCP_SECRET_BUCKET: secretBucket,
        GCP_ROLES_BUCKET: rolesBucket,
        GCP_KMS_SECRET_BUCKET_KEY_LOCATION: secretBucketKeyLocation,
        GCP_KMS_SECRET_BUCKET_KEY_RING: secretBucketKeyRing,
        GCP_COMPUTE_URL_ID: computeUrlId,
        ...env,
      }).reduce((string, [key, value]) => (string += `${key}=${value},`), "")}`,
      `--labels=${Object.entries({
        procedure,
        hash: operationHash,
        ...labels,
      }).reduce((string, [key, value]) => (string += `${key}=${value},`), "")}`,
    ],
  };
};
